// AnimationPanel.java
//
// Informatics 45 Spring 2010
// Code Example: Ball Animation (Version 1)
//
// This panel is used to draw the current state of the animation at any
// given time.  Since the state of our animation can be drawn as a single
// filled oval, the only trick is to decide the pixel coordinates at which
// the oval should be drawn, since the animation uses a fractional coordinate
// system (0.0 through 1.0 on both the x- and y-axes).  This transformation
// is implemented in the two convertXXX() methods below.

package inf45.spring2010.examples.animation1.gui;

import inf45.spring2010.examples.animation1.model.AnimationState;
import java.awt.*;
import javax.swing.*;


public class AnimationPanel extends JPanel
{
	private AnimationState currentState;
	
	
	public AnimationPanel()
	{
		currentState = null;
		setBackground(Color.BLACK);
	}


	public void updateState(AnimationState newState)
	{
		currentState = newState;
	}
	
	
	// This method is called whenever the animation panel (or any part of
	// it) needs to be repainted.  The bulk of the work is in converting
	// coordinates from the animation-related fractional coordinates to
	// pixel coordinates.  Once that's done, drawing the animation is a
	// simple matter of setting the color and filling an oval in the
	// appropriate location.
	public void paint(Graphics g)
	{
		super.paint(g);
		
		if (currentState == null)
		{
			return;
		}

		Graphics2D g2d = (Graphics2D) g;

		g2d.setRenderingHint(
			RenderingHints.KEY_ANTIALIASING,
			RenderingHints.VALUE_ANTIALIAS_ON);

		int centerPixelX = convertXToPixelX(currentState.getBallCenterX());
		int centerPixelY = convertYToPixelY(currentState.getBallCenterY());
		
		int radiusX = convertXToPixelX(currentState.getBallRadius());
		int radiusY = convertYToPixelY(currentState.getBallRadius());

		
		// To draw an oval, we need to know two things: 
		//
		// 1. The x- and y-coordinates of the top left corner of an invisible
		//    box that surrounds the oval.
		// 2. The width and height of that invisible box.
		//
		// The oval is, then, dutifully drawn exactly within the boundaries of
		// the invisible box.
		//
		// What we have is the x- and y-coordinates of the center of the oval,
		// along with the radius of the oval.  A couple of simple formulas can
		// be used to do the appropriate conversion.

		int topLeftPixelX = centerPixelX - radiusX;
		int topLeftPixelY = centerPixelY - radiusY;

		
		// Finally, we draw our oval.
		g2d.setColor(Color.GREEN);
		g2d.fillOval(topLeftPixelX, topLeftPixelY, radiusX * 2, radiusY * 2);
	}
	

	// convertXToPixelX() converts a fractional x-coordinate into a pixel
	// coordinate.  One upside of the fractional coordinate system is that
	// we can do the conversion simply by multiplying the fractional
	// coordinate by the width of the panel, since, for example, a
	// fractional coordinate of 0.5 should be translated to a pixel
	// coordinate that's halfway across the panel (left to right).
	private int convertXToPixelX(double x)
	{
		return (int) (x * getWidth());
	}
	

	// convertYToPixelY() converts a fractional y-coordinate into a pixel
	// coordinate, using a formula similar to the one used for the
	// x-coordinate.
	private int convertYToPixelY(double y)
	{
		return (int) (y * getHeight());
	}
}
